var swiper = function(Vue) {
  Vue.directive('swiper', {
    bind: function(el, binding, vnode) {
      var touchType = binding.arg //传入的模式 press swipeRight swipeLeft swipeTop swipeDowm Tap
      var timeOutEvent = 0
      var direction = ''
      //滑动处理
      var startX, startY
      //返回角度
      function GetSlideAngle(dx, dy) {
        return (Math.atan2(dy, dx) * 180) / Math.PI
      }
      //根据起点和终点返回方向 1：向上，2：向下，3：向左，4：向右,0：未滑动
      function GetSlideDirection(startX, startY, endX, endY) {
        var dy = startY - endY
        var dx = endX - startX
        var result = 0
        //如果滑动距离太短
        if (Math.abs(dx) < 2 && Math.abs(dy) < 2) {
          return result
        }
        var angle = GetSlideAngle(dx, dy)
        if (angle >= -45 && angle < 45) {
          result = 'swiperight'
        } else if (angle >= 45 && angle < 135) {
          result = 'swipeup'
        } else if (angle >= -135 && angle < -45) {
          result = 'swipedown'
        } else if (
          (angle >= 135 && angle <= 180) ||
          (angle >= -180 && angle < -135)
        ) {
          result = 'swipeleft'
        }
        return result
      }
      el.addEventListener(
        'touchstart',
        function(ev) {
          startX = ev.touches[0].pageX
          startY = ev.touches[0].pageY
          //判断长按
          timeOutEvent = setTimeout(() => {
            timeOutEvent = 0
            if (touchType === 'press') {
              binding.value()
            }
          }, 500)
        },
        false
      )
      el.addEventListener('touchmove', function(ev) {
        clearTimeout(timeOutEvent)
        timeOutEvent = 0
      })
      el.addEventListener(
        'touchend',
        function(ev) {
          var endX, endY
          endX = ev.changedTouches[0].pageX
          endY = ev.changedTouches[0].pageY
          direction = GetSlideDirection(startX, startY, endX, endY)
          clearTimeout(timeOutEvent)
          switch (direction) {
            case 0:
              break
            case 'swipeup':
              if (touchType === 'swipeup') {
                binding.value()
              }
              break
            case 'swipedown':
              if (touchType === 'swipedown') {
                binding.value()
              }
              break
            case 'swipeleft':
              if (touchType === 'swipeleft') {
                binding.value()
              }
              break
            case 'swiperight':
              if (touchType === 'swiperight') {
                binding.value()
              }
              break
            default:
          }
        },
        false
      )
    }
  })
}
module.exports = swiper

const focus = function(Vue) {
  Vue.directive('focus', {
    inserted: function(el) {
      const allFocusEls = getAllFocusEls(el) // 获取需要聚焦的input元素组
      let current = getTargetIndex(el, allFocusEls)
      if (!current || current < 0 || current >= allFocusEls.length) {
        // 如果没有设置data-current,或者current的数值范围不符合要求,则默认聚焦到第一个输入框
        current = 0
      }
      const currentEl = allFocusEls[current]

      autoFocus(currentEl)

      const focusCtrl = function(el) {
        const action = el.dataset.action
        const allFocusEls = getAllFocusEls(el)
        const focusLen = allFocusEls.length
        let current = getTargetIndex(el, allFocusEls)
        switch (action) {
          case 'next': // 如果action为next,则聚焦到下一个输入框
            if (current >= focusLen - 1) {
              current = focusLen - 1
            } else {
              current++
            }
            autoFocus(allFocusEls[current])
            break
          case 'prev': // 如果action为prev,则聚焦到上一个输入框
            if (current <= 0) {
              current = 0
            } else {
              current--
            }
            autoFocus(allFocusEls[current])
            break
          case 'first': // 如果为first,则聚焦到第一个输入框
            current = 0
            autoFocus(allFocusEls[current])
            break
          case 'last': // 如果为last,则聚焦到最后一个输入框
            current = focusLen - 1
            autoFocus(allFocusEls[current])
            break
          case 'jump': // 如果为jump,则获取focusIndex,跳转到对应的输入框
            if (current >= 0 && current < focusLen) {
              autoFocus(allFocusEls[current])
            }
            break
        }
      }
    }
  })
}

module.exports = focus

const drag = function(Vue) {
  Vue.directive('drag', {
    bind: function(el, binding) {
      let isDrag = true
      let tempX = 0
      let x = 0
      let tempY = 0
      let y = 0

      el.ontouchstart = e => {
        isDrag = true
        tempX = parseInt(el.style.left + 0)
        tempY = parseInt(el.style.top + 0)
        x = e.touches[0].pageX
        y = e.touches[0].pageY
      }

      el.ontouchmove = e => {
        if (isDrag) {
          let curX = tempX + e.touches[0].pageX - x
          let curY = tempY + e.touches[0].pageY - y
          let height = document.documentElement.clientHeight
          if (binding.value) {
            height = el.parentElement.clientHeight
          }
          let clientWidth =
            document.documentElement.clientWidth - el.clientWidth - 2
          let clientHeight = height - el.clientHeight - 2
          //边界判断
          curX = curX < 0 ? 0 : curX
          curY = curY < 0 ? 0 : curY
          curX = curX < clientWidth ? curX : clientWidth
          curY = curY < clientHeight ? curY : clientHeight

          el.style.left = curX + 'px'
          el.style.top = curY + 'px'
          //阻止浏览器继续处理触摸(和鼠标)事件。
          e.preventDefault()
        }
      }
      el.ontouchend = e => {
        isDrag = false
      }
    }
  })
}

module.exports = drag

const imgerror = function(Vue) {
  Vue.directive('imgerror', async function(el, binding) {
    let imgURL = binding.value //获取图片地址
    let defaultURL = el.getAttribute('default-img') //获取默认图片地址
    if (imgURL) {
      let exist = await imageIsExist(imgURL)
      if (exist) {
        el.setAttribute('src', imgURL)
      } else {
        if (defaultURL) {
          el.setAttribute('src', defaultURL)
        }
      }
    }
  })
  const imageIsExist = function(url) {
    return new Promise(resolve => {
      var img = new Image()
      img.onload = function() {
        if (this.complete == true) {
          resolve(true)
          img = null
        }
      }
      img.onerror = function() {
        resolve(false)
        img = null
      }
      img.src = url
    })
  }
}
module.exports = imgerror
